#ifndef __4DrobotServer_config_h__
#define __4DrobotServer_config_h__

//Note: This isn't a proper header file, as it contains implementations (variable = value).
//      It can therefore only be included in one compilation unit.
//      This is OK as long as we stay within Arduino studio and only use .ino files for code,
//      which are concatonated in alphabetical order.

//Note on Arduino pins used for Ethernet shield,
// AVOID THESE PINS:
// 4/10/11/12/13 (UNO)
// 50/51/52/10/4 (Mega)

// ***** HARDWARE SIMULATION (FOR DEV) *********

//Uncomment to simulate hardware
#define DUMMY_TEMP
#define DUMMY_SERVO
#define DUMMY_STEPPER

// ***** NETWORK CONFIG ************************

// MAC address for the Arduino on the plasma lens
//byte mac[] = {0xA8, 0x61, 0x0A, 0xAE, 0x84, 0xED};

// MAC address for the Arduino on the CLIC structure
//byte mac[] = {0xA8, 0x61, 0x0A, 0xAE, 0x84, 0xEE};

// MAC address for Kyrre's test board
byte mac[] = {0xA8, 0x61, 0x0A, 0xAE, 0x72, 0xF7};

//To use DHCP or not to use DHCP, that's the question.
// Comment out to use static IP configuration
#define USE_DHCP

#ifndef USE_DHCP
IPAddress ip(192, 168, 1, 177);
IPAddress myDns(192, 168, 1, 1);
IPAddress gateway(192, 168, 1, 1);
IPAddress subnet(255, 255, 0, 0);
#endif

const int    telnet_port = 23;

//Number of connection 'slots'
const size_t telnet_numConnections = MAX_SOCK_NUM;

//This quickly takes a LOT of memory!
//The total for MEGA is 8k, and we also need some stack space!
//Note: It's possible to reduce the number of connection slots.
const size_t telnet_bufflen = 256;
const size_t output_bufflen = 2048;

// ***** TEMPERATURE SENSOR CONFIG *************

const uint8_t temp_oneWire_pin           =   51; // <-- Conflict with ETHERNET!
const size_t  temp_numSensors            =    1;
const unsigned long temp_update_interval = 1000; //[ms]

// ***** SAMPLE GRABBER SERVO CONFIG ***********

// TODO: Tune stepWait, positions
const int grabber_servo_pin    =   9;
const int grabber_closed       =  86; //[deg]
const int grabber_open         = 130; //[deg]
const int grabber_min          =   0; //[deg]
const int grabber_max          = 180; //[deg]
const int grabber_stepWait     =  50; //[ms]

// ***** STEPPER CONFIGS ***********************

const size_t stepper_numAxis                 = 3;

//Array indices of the 3 axis
#define STEPPER_X 0
#define STEPPER_Y 1
#define STEPPER_Z 2

const char    stepper_axnames[]              = {'X','Y','Z'};

const uint8_t stepper_dir_pin[]              = {32,36,40}; //Stepper direction pins (HIGH=positive, LOW=negative)
const uint8_t stepper_step_pin[]             = {33,37,41}; //Stepper pulse-to-step pins (pulse HIGH, otherwise low)

//Define level for stepper_dir_pin[] to count steps in forward direction
// Used to define the Z axis as zero when on top, so that we can zero it first on top switch,
// then leave it on the limit switch while zeroing the others (safe).
// TODO: Verify!!
const bool    stepper_forward[]              = {LOW,LOW,HIGH};
const bool    stepper_backward[]             = {HIGH,LOW,LOW};

//TODO: PIN4 ALSO IN USE BY ETHERNET
//TODO: Verify reproducibility of zeroing, we may need debounce?
const uint8_t stepper_switchMAX_pin[]         = {2,5,7}; //positive steps limit switch pins
const uint8_t stepper_switchMIN_pin[]         = {3,4,6}; //negative steps limit switch pins
//TODO: Verify active/inactive -> HIGH/LOW
//      Note: Needs pulldown/pullup resistor
//      Note: Probably safest to define on-limit = open, so if a switch fails the axis is safe (but stopped)
const bool    stepper_switch_active          = HIGH;    //On limit switch when pin active
const bool    stepper_switch_inactive        = LOW;     //On limit switch when pin inactive

//TODO: Tune timings
const unsigned int  stepper_onTime           = 25;  // [us] How long to hold the step pin HIGH
const unsigned int  stepper_delay_fast       = 25;  // [us] How long to wait between steps when moving fast
const unsigned int  stepper_delay_slow       = 100; // [us] How long to wait between steps when moving slow
const unsigned int  stepper_accelerate_steps = 100; // When accelerating, how many steps to use for ramp

// When at high speed, ramp down to slow speed when
// pos < accelerate_steps+slowzone or pos > (rail_length-accelerate_steps-slowzone)
//TODO: Tune distances
const unsigned long int  stepper_rail_length[]    = {1000,1000,1000}; // [steps] Total length of the rails
const unsigned long int  stepper_slowzone_steps   = 50;               // [steps] Take it easy when approaching limit switches

//Interlocks
const long int           stepper_Zpos_unlockXY    = 200;              // [steps] Only allow XY movements when
                                                                 //         Zpos > Zpos_unlockXY and Z axis
                                                                 //         in absolute mode
const unsigned long int  stepper_onlimit_steps    = 10;               // [steps] Number of steps to allow movement
                                                                 //         per command when a limit switch
                                                                 //         is active
// ***** INTERNAL CONFIGS **********************

// Uncomment to include extra debug messages about parser to Serial
//#define PARSER_DEBUG

#endif
