void setup_stepper() {
  // Set pin modes
  for (size_t i=0; i<stepper_numAxis; i++) {
    pinMode(stepper_dir_pin[i], OUTPUT);
    pinMode(stepper_step_pin[i], OUTPUT);
    
    pinMode(stepper_switchMAX_pin[i], INPUT);
    pinMode(stepper_switchMIN_pin[i], INPUT);
  }
}

void stepper_control() {
  if(not stepper_go) {
    return;
  }
  
  //Check XY interlock
  // error if we try to move locked axis
  if( (stepper_go_axis != STEPPER_Z) and
      ( (not stepper_mode_absolute[STEPPER_Z]) or
        (stepper_pos[STEPPER_Z] <= stepper_Zpos_unlockXY)
      )
    ) {
    //We are trying to move a locked axis
    bufferWrite(output_buff, sizeof(output_buff), output_buffCount, F("< STEPPER ERROR XY INTERLOCK\n"));
    goto resetGO;
  }

  //Move!
  bufferWrite(output_buff, sizeof(output_buff), output_buffCount, F("< STEPPER MOVING "));
  bufferWrite(output_buff, sizeof(output_buff), output_buffCount, stepper_axnames[stepper_go_axis]);
  bufferWrite(output_buff, sizeof(output_buff), output_buffCount, F("...\n"));
  output_buff_flush();

  //Task: Zeroseek an axis
  if (stepper_go_zero) {
    
    //Always move in negative direction when zeroing
    digitalWrite(stepper_go_axis, stepper_backward[stepper_go_axis]);

    //Move until we trigger the low interlock switch
    bool onMAX = (digitalRead(stepper_switchMAX_pin[stepper_go_axis]) == stepper_switch_active);
    bool onMIN = (digitalRead(stepper_switchMIN_pin[stepper_go_axis]) == stepper_switch_active);
    int  onLimitCount = 0;

    while (not onMIN) {
      //Check the other-end switch that we aren't moving in the wrong direction or stuck
      if(onMAX) {
        onLimitCount++;
      }
      if(onLimitCount > stepper_onlimit_steps) {
        bufferWrite(output_buff, sizeof(output_buff), output_buffCount, F("< STEPPER ERROR LIMIT SWITCH\n"));
        goto resetGO;
      }

      //Move (slowly)
      digitalWrite(stepper_go_axis,HIGH); 
      delayMicroseconds(stepper_onTime);
      digitalWrite(stepper_go_axis,LOW); 
      delayMicroseconds(stepper_delay_slow);

      stepper_pos[stepper_go_axis] -= 1L;

      //Poll switches
      onMAX = (digitalRead(stepper_switchMAX_pin[stepper_go_axis]) == stepper_switch_active);
      onMIN = (digitalRead(stepper_switchMIN_pin[stepper_go_axis]) == stepper_switch_active);
    }

    bufferWrite(output_buff, sizeof(output_buff), output_buffCount, F("< STEPPER ZEROSEEK MOVED "));
    char buff[13];
    snprintf(buff,sizeof(buff), "%+011ld\n",stepper_pos[stepper_go_axis]);
    bufferWrite(output_buff, sizeof(output_buff), output_buffCount, buff);

    //Movement to switch completed successfully,
    stepper_pos[stepper_go_axis]           = 0L;
    stepper_mode_absolute[stepper_go_axis] = true;
  }

  //Task: Move to given position (either relative or absolute)
  else {
    int stepping = 0;
    if (stepper_goto > stepper_pos[stepper_go_axis]) {
      digitalWrite(stepper_go_axis, stepper_forward[stepper_go_axis]);
      stepping = 1;
    }
    else if(stepper_goto < stepper_pos[stepper_go_axis]) {
      digitalWrite(stepper_go_axis, stepper_backward[stepper_go_axis]);
      stepping = -1;
    }
    // Note: No 'else' needed; if they are equal, the while loop doesn't iterate.

    //Move until goto==pos or we trigger an interlock switch stepper_onlimit_steps times
    bool onMAX = (digitalRead(stepper_switchMAX_pin[stepper_go_axis]) == stepper_switch_active);
    bool onMIN = (digitalRead(stepper_switchMIN_pin[stepper_go_axis]) == stepper_switch_active);

    unsigned long int onLimitCount = 0L;

    while (stepper_pos[stepper_go_axis] != stepper_goto) {
      //Check limit switches
      if(onMAX or onMIN) {
        onLimitCount++;
      }
      if(onLimitCount > stepper_onlimit_steps) {
        bufferWrite(output_buff, sizeof(output_buff), output_buffCount, F("< STEPPER ERROR LIMIT SWITCH\n"));
        goto resetGO;
      }

      //Move (slowly)
      //TODO: Implement acceleration
      digitalWrite(stepper_go_axis,HIGH); 
      delayMicroseconds(stepper_onTime);
      digitalWrite(stepper_go_axis,LOW); 
      delayMicroseconds(stepper_delay_slow);
      stepper_pos[stepper_go_axis] += stepping;

      //Poll switches
      onMAX = (digitalRead(stepper_switchMAX_pin[stepper_go_axis]) == stepper_switch_active);
      onMIN = (digitalRead(stepper_switchMIN_pin[stepper_go_axis]) == stepper_switch_active);
    }
  }
    

  //Reset the flags which tells it where to go
  //Can also jump (GOTO) here directly on error
  resetGO: //Yep this is a GOTO label.
           // Please read XKCD297 & XKCD292

  //Message '< STEPPER GO FINISHED AXIS <X/Y/Z> POS <POSITION>\n'
  bufferWrite(output_buff, sizeof(output_buff), output_buffCount, F("< STEPPER GO FINISHED AXIS "));
  bufferWrite(output_buff, sizeof(output_buff), output_buffCount, stepper_axnames[stepper_go_axis]);
  bufferWrite(output_buff, sizeof(output_buff), output_buffCount, F(" POS "));
  char buff[13];
  snprintf(buff,sizeof(buff), "%+011ld\n",stepper_pos[stepper_go_axis]);
  bufferWrite(output_buff, sizeof(output_buff), output_buffCount, buff);

  stepper_go      = false;
  stepper_go_zero = false;
  stepper_go_axis = 0;
  stepper_goto    = 0L;

  return;
}
